
#include <avr/io.h>

/******************************** FONCTIONS I2C *******************************/

// Initialisation avec calcul des predivision I2C
// fcpu : frequence du microcontroleur
// ftwi : frequence souhaitee pour le bus I2C
void I2C_Init(long fcpu, long ftwi);

// Initialisation I2C master
// rate : valeur du registre TWBR pour slection du dbit
// prescaler : valeur des bits du registre TWSR pour slection du prescaler
// ces paramtres dterminent la frquence de l'horloge I2C
void I2C_MasterInit(char rate, char prescaler);

// Envoi de la squence de start + adresse de l'esclave
// adresse : adresse de l'esclave avec lequel dmarrer la communication
// mode= 1 si lecture et 0 si criture
int I2C_MasterStart(char adresse, int mode);

// Envoi de la squence de restart + adresse de l'esclave
// adresse : adresse de l'esclave avec lequel dmarrer la communication
// mode= 1 si lecture et 0 si criture
int I2C_MasterRestart(char adresse, int mode);

// Envoi de la squence stop
void I2C_MasterStop(void);

// Ecriture d'un octet sur le bus
// donnee : l'octet  crire
int I2C_MasterWrite(char c);

// Lecture d'un octet sur le bus avec acknowledge  la fin
// pour indiquer  l'esclave que la lecture n'est pas termine
// Renvoie la donne lue
char I2C_MasterReadNext(void);

// Lecture d'un octet sur le bus sans acknowledge  la fin
// pour indiquer  l'esclave que la lecture est termine
// (doit tre suivi par un stop)
// Renvoie la donne lue
char I2C_MasterReadLast(void);

// Fonction  usage interne
void I2C_MasterWait(void);

// Lecture d'une valeur dans registre (reg) sur peripherique (addr)
char I2C_MasterGetReg(char addr, char reg);

// Ecriture d'une valeur dans registre (reg) sur peripherique (addr)
void I2C_MasterSetReg(char addr, char reg, char val);
