#include <string.h>

#include "UsrLib.h"

#define BIT_LCD_RS   	0     // bit du port sur lequel est connecte la pin RS du LCD		// adapte pour IO-Expander facon Arduino
#define BIT_LCD_WR   	1     // bit du port sur lequel est connecte la pin WR du LCD		// inutilise
#define BIT_LCD_E   	2     // bit du port sur lequel est connecte la pin E du LCD		//
#define BIT_LCD_BL   	3     // bit du port sur lequel est connecte la pin BL du LCD		// backlight (optionnel)
#define BIT_LCD_D4   	4     // bit du port sur lequel est connecte la pin D4 du LCD		//
#define BIT_LCD_D5   	5     // bit du port sur lequel est connecte la pin D5 du LCD		//
#define BIT_LCD_D6   	6     // bit du port sur lequel est connecte la pin D6 du LCD		//
#define BIT_LCD_D7   	7     // bit du port sur lequel est connecte la pin D7 du LCD		//

#define PORT_LCD(x)		port x; I2C_MasterSetReg(lcd_i2c_adr, 0, port);		// adaptation pour IO-Expander I2C

#define LCD_CMD_EFF        	  0x01   // Commande d'effacement LCD
#define LCD_CMD_HOME      	  0x02   // Commande de renvoi du curseur  la position initiale
#define LCD_CMD_ENTRY_MODE    0x04   // Commande de slection du mode d'insertion
#define LCD_BIT_ENTRY_SHIFT   0      // Position bit de slection inversion/non inversion
#define LCD_BIT_ENTRY_INC     1      // Position bit de slection 									
							  		 // incrmentation/dcrementation curseur
#define LCD_CMD_DISPLAY 	  0x08   // Commande de gestion de l'affichage LCD
#define LCD_BIT_DISP_DISP     2      // Position bit d'activation/dsactivation de l'affichage
#define LCD_BIT_DISP_CURS     1      // Position bit d'activation/dsactivation curseur
#define LCD_BIT_DISP_BLINK    0      // Position bit d'activation/dsactivation clignotement curseur
#define LCD_CMD_SHIFT      	  0x10   // Commande de dplacement affichage/curseur
#define LCD_BIT_SHIFT_TYPE    3      // Position bit de type de dplacement
#define LCD_BIT_SHIFT_SENS    2      // Position bit de sens de dplacement
#define LCD_CMD_SET_DDRAM     0x80   // Commande d'affectation de l'adresse DDRAM
#define LCD_LIGNE_1           0x00   // Adresse ligne 1
#define LCD_LIGNE_2           0x40   // Adresse ligne 2
#define LCD_CMD_SET_CGRAM     0x40   // Commande d'affectation de l'adresse CGRAM

// Envoi au LCD du caractre pass en paramtre
void LCD_I2C_SendChar(char caractere);
   
// Envoi au LCD de la commande passe en paramtre
void LCD_I2C_SendCommand(char commande);

// Initialisation du LCD
void LCD_I2C_Init(int i2c_adr);

// Effacement du LCD
void LCD_I2C_Erase(void);

// Activation / desactivation du backlight (facultatif)
void LCD_I2C_BackLight(char stat);

// Renvoi du curseur en haut  gauche
void LCD_I2C_Home(void);

// Slection du mode d'insertion pour le LCD
// direction_curseur : 1 = incrmentation - 0 = dcrmentation
// inversion_affichage : 1 = inversion - 0 = non inversion
void LCD_I2C_InsertMode(int direction_curseur,int inversion_affichage);

// Configuration affichage LCD
// affichage_actif : 1 = actif - 0 = inactif
// curseur_actif : 1 = actif - 0 = inactif
// clignotement_actif : 1 = actif - 0 = inactif
void LCD_I2C_Config(int affichage_actif,int curseur_actif,int clignotement_actif);

// Dplacement affichage ou curseur
// type : 1 = affichage - 0 = curseur
// sens : 1 = droite - 0 = gauche
void LCD_I2C_Move(int type,int sens);

// Dplacement du curseur de n caractres vers la droite
// nombre_caracteres : nombre de caracteres de dplacement vers la droite
void LCD_I2C_MoveRight(int nombre_caracteres);

// Dplacement du curseur de n caractres vers la gauche
// nombre_caracteres : nombre de caracteres de dplacement vers la gauche
void LCD_I2C_MoveLeft(int nombre_caracteres);

// Dplacement de n caractres vers la droite des donnes  partir du curseur
// nombre_caracteres : nombre de caracteres de dplacement vers la droite
void LCD_I2C_ShiftRight(int nombre_caracteres);

// Dplacement de n caractres vers la gauche des donnes  partir du curseur
// nombre_caracteres : nombre de caracteres de dplacement vers la gauche
void LCD_I2C_ShiftLeft(int nombre_caracteres);

// Ecriture d'une chane de caractre sur le LCD
// chaine : le pointeur sur la chane de caractres  envoyer
void LCD_I2C_Write(char *chaine);

// Dplacement du curseur
// y : numro de la ligne (1 ou 2)
// x : numro du caractre (1  16)
void LCD_I2C_MoveCursor(int y,int x);

// Affiche une variale de type entier sur l'cran LCD
// entier : l'entier  afficher sur l'cran
void LCD_I2C_ShowLong(long entier);

// Affiche une variable de type double  l'cran avec un nombre de dcimales dfini
// vdouble : la valeur dcimale  afficher
// nb_decimales : le nombre de dcimales  afficher aprs la virgule
void LCD_I2C_ShowDouble(double vdouble, int nb_decimales);


