#include <htc.h>

// Librairie pour afficheur LCD 16x2 + IO-Expander sur bus I2C

#include "ladder.h"
#include "UsrLib.h"
#include "I2cLib.h"
#include "LcdI2clib.h" // Fichier header librairie LCD

unsigned char port = 0; // adaptation pour IO-Expander I2C

// Gre l'criture d'un octet vers le LCD
void LCD_I2C_Send(char donnee, int type)
{
    // Activation de RS si type = 1 (donnee)
    PORT_LCD(|= (type << BIT_LCD_RS))
    // Ecriture des 4 MSB
    LCD_I2C_Send4msb(donnee);
    // Ecriture des 4 LSB
    LCD_I2C_Send4msb(donnee << 4);
    // Desactivation RS
    PORT_LCD(&= ~(1 << BIT_LCD_RS))
    // Attente execution
    delay_ms(6);
}

// Gre l'enable du LCD
void LCD_I2C_Enable(void)
{
    PORT_LCD(|= (1 << BIT_LCD_E)) // E  1
                                  //   	asm("NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;NOP;");
    delay_us(10);
    PORT_LCD(&= ~(1 << BIT_LCD_E)) // E  0
}

// Gre l'criture des 4 MSB vesr le LCD
void LCD_I2C_Send4msb(char donnee)
{
    // Reset bit de donnes
    PORT_LCD(&= ~((1 << BIT_LCD_D4) | (1 << BIT_LCD_D5) | (1 << BIT_LCD_D6) | (1 << BIT_LCD_D7)))

    // Ecriture des 4 MSB
    if(donnee & (1 << 4)) {
        PORT_LCD(|= (1 << BIT_LCD_D4)) // Bit D4  1
    }

    if(donnee & (1 << 5)) {
        PORT_LCD(|= (1 << BIT_LCD_D5)) // Bit D5  1
    }

    if(donnee & (1 << 6)) {
        PORT_LCD(|= (1 << BIT_LCD_D6)) // Bit D6  1
    }

    if(donnee & (1 << 7)) {
        PORT_LCD(|= (1 << BIT_LCD_D7)) // Bit D7  1
    }

    // Validation donnee par basculement de l'enable
    LCD_I2C_Enable();
}

// Efface le contenu de l'ecran LCD
void LCD_I2C_Erase(void)
{
    LCD_I2C_SendCommand(LCD_CMD_EFF);
}

// Renvoie le curseur  la position initiale
void LCD_I2C_Home(void)
{
    LCD_I2C_SendCommand(LCD_CMD_HOME);
}

// Activation / desactivation du backlight (facultatif)
void LCD_I2C_BackLight(char stat)
{
    if(stat) {
        PORT_LCD(|= (1 << BIT_LCD_BL))
    } else {
        PORT_LCD(&= ~(1 << BIT_LCD_BL))
    }
}

// Slection du mode d'insertion des caractres sur le LCD
void LCD_I2C_InsertMode(int direction_curseur, int inversion_affichage)
{
    LCD_I2C_SendCommand(LCD_CMD_ENTRY_MODE                             //
                        | (inversion_affichage << LCD_BIT_ENTRY_SHIFT) //
                        | (direction_curseur << LCD_BIT_ENTRY_INC)     //
    );
}

// Configuration affichage LCD
void LCD_I2C_Config(int affichage_actif, int curseur_actif, int clignotement_actif)
{
    LCD_I2C_SendCommand(LCD_CMD_DISPLAY                              //
                        | (affichage_actif << LCD_BIT_DISP_DISP)     //
                        | (curseur_actif << LCD_BIT_DISP_CURS)       //
                        | (clignotement_actif << LCD_BIT_DISP_BLINK) //
    );
}

// Dplacement affichage ou curseur
void LCD_I2C_Move(int type, int sens)
{
    LCD_I2C_SendCommand(LCD_CMD_SHIFT                  //
                        | (type << LCD_BIT_SHIFT_TYPE) //
                        | (sens << LCD_BIT_SHIFT_SENS) //
    );
}

// Dplacement du curseur de n caractres vers la droite
void LCD_I2C_MoveRight(int nombre_caracteres)
{
    int i;

    for(i = 0; i < nombre_caracteres; i++)
        LCD_I2C_Move(0, 1);
}

// Dplacement du curseur de n caractres vers la gauche
void LCD_I2C_MoveLeft(int nombre_caracteres)
{
    int i;

    for(i = 0; i < nombre_caracteres; i++)
        LCD_I2C_Move(0, 0);
}

// Dplacement de n caractres vers la droite des donnes  partir du curseur
void LCD_I2C_ShiftRight(int nombre_caracteres)
{
    int i;

    for(i = 0; i < nombre_caracteres; i++)
        LCD_I2C_Move(1, 1);
}

// Dplacement de n caractres vers la gauche des donnes  partir du curseur
void LCD_I2C_ShiftLeft(int nombre_caracteres)
{
    int i;

    for(i = 0; i < nombre_caracteres; i++)
        LCD_I2C_Move(1, 0);
}

// Ecriture d'une chane de caractre sur le LCD
void LCD_I2C_Write(char *chaine)
{
    int i;

    if(chaine == 0) // Si le pointeur pass en paramtre n'est pas valide
        return;     // => sortie de la fonction

    for(i = 0; i < strlen(chaine); i++)
        LCD_I2C_SendChar(chaine[i]);
}

// Dplacement du curseur
void LCD_I2C_MoveCursor(int y, int x)
{
    int adresse;

    if((y < 1) || (y > 2) || (x < 1) || (x > 16)) {
        return;
        // Si le numro de caractre/ligne n'est pas bon => sortie de la fonction
    }

    // Ligne 1
    if(y == 1) {
        adresse = LCD_LIGNE_1 + (x - 1);
    }
    // Ligne 2
    else {
        adresse = LCD_LIGNE_2 + (x - 1);
    }

    LCD_I2C_SendCommand(LCD_CMD_SET_DDRAM | adresse);
}

/*
// Affiche une variale de type entier sur l'cran LCD
void LCD_I2C_ShowLong(long entier)
	{
   	int compteur = 0;
   	char caracteres[10];  // Un entier long comporte un maximum de 10 digits  
   
   	// Si l'entier vaut 0, on l'affiche tout de suite
   	if(entier == 0)
   		{
      	LCD_I2C_SendChar('0');
   		}   
   	else
   		{
      	// Gestion de l'affichage du signe moins dans le cas d'un nombre ngatif
      	if(entier < 0)
      		{
        	LCD_I2C_SendChar('-');
        	entier = -entier;   // Repassage du nombre en positif
      		}
      
      	// Construction de la chane de caractres correspondante
      	// Principe: divisions successives par 10 pour rcuprer chaque digit
      	// jusqu' arriver  0.
      	while((entier>0) && (compteur <= 10))
      		{
        	caracteres[compteur] = entier % 10;  // Rcupration du digit actuel
        	entier /= 10;   // Passage au digit suivant
        	compteur++;
      		}
      
      	// Affichage de la chane (on commence par les derniers digit
      	// puisque la construction s'est faites dans le sens inverse)
      	for(;compteur>0; compteur--)
      		{
        	LCD_I2C_SendChar('0' + caracteres[compteur-1]);
      		}
   		}
	}

// Affiche une variable de type double  l'cran avec un nombre de dcimales dfini
void LCD_I2C_ShowDouble(double vdouble, int nb_decimales)
	{
   	unsigned long fraction;
   	unsigned long fraction_successives;
   	unsigned long multiple = 1;
   	int nb_decimales_affichees = nb_decimales - 1;

   	// Etape 1 : Affichage de la valeur entire
   	LCD_I2C_ShowLong((long) vdouble);

   	// Etape 2 : Affichage de la partie dcimale
   	// Principe : Multiplication successive par 10 pour construire le nombre entier
   	// correspondant aux dcimales

   	if (nb_decimales > 0)
   		{
      	// Affichage de la virgule
      	LCD_I2C_SendChar(',');
      
      	// Rcupration du premier diviseur
      	while(nb_decimales--)
      		{
        	multiple *= 10;
      		}
      
      	// Toute la partie dcimale est transforme en partie entire      
      	// Nombre positif
      	if (vdouble >= 0)
      		{
        	fraction = (vdouble - (int) vdouble) * multiple;
      		}
      	// Nombre ngatif
      	else
      		{
        	fraction = ((int) vdouble - vdouble) * multiple;      
      		}
      
      	fraction_successives = fraction;
      
      	// Vrification si besoin de rajouter des zros aprs la virgule
      	// avant d'afficher la partie dcimale
      	while(fraction_successives /= 10)
      		{
        	nb_decimales_affichees--;
      		}
      
      	// Affichage des zros si besoin
      	while(nb_decimales_affichees--)
      		{
        	LCD_I2C_SendChar('0');
      		}
      
      	// Affichage de la partie dcimale
      	LCD_I2C_ShowLong((long) fraction);
   		}
	}
*/
